@@ -43,7 +43,6 @@ OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #include "primer3.h"
 #include "boulder_input.h"
 
-/* Type definitions. */
 /* #define's */
 
 /* 
@@ -57,9 +56,11 @@ OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #define OOM_ERROR OOM_STMT1, OOM_STMT2
 
 #ifndef MAX_PRIMER_LENGTH
-Important note: MAX_PRIMER_LENGTH must be defined in
-the makefile, because its value must be <= the
-value of DPAL_MAX_ALIGN.
+#error "Define MAX_PRIMER_LENGTH in Makefile..."
+  /* to ensure that MAX_PRIMER_LENGTH <= DPAL_MAX_ALIGN. */
+#endif
+#if (MAX_PRIMER_LENGTH > DPAL_MAX_ALIGN) 
+#error "MAX_PRIMER_LENGTH must be <= DPAL_MAX_ALIGN"
 #endif
 
 #define MAX_NN_TM_LENGTH 36 /* The maxium length for which to use the
@@ -149,13 +150,21 @@ static double p_obj_fn(const primer_args *, primer_rec *, int );
 static void   oligo_compl(primer_rec *, const primer_args *, seq_args *,
 			  oligo_type, const dpal_args *, 
 			  const dpal_args*, const dpal_args *);
-static void   oligo_mispriming(primer_rec *, const primer_args *, seq_args *,
-			       oligo_type, const dpal_args *);
+static void   oligo_mispriming(primer_rec *,
+			       const primer_args *,
+			       seq_args *,
+			       oligo_type, 
+			       const dpal_args *);
 static int    pair_repeat_sim(primer_pair *, const primer_args *);
 static void   boulder_print_oligos(const primer_args *, 
-					 const seq_args *, int, oligo_type);
+				   const seq_args *, int, oligo_type);
 static void   free_repeat_sim_score(int, int, int);
 
+/* edited by T. Koressaar for lowercase masking:  */
+static void   check_if_lowercase_masked(const int position,
+					const char *sequence,
+					primer_rec *h);
+
 /* Global static variables. */
 static const char * copyright[] = {
 "",
@@ -294,13 +303,11 @@ main(argc,argv)
 	input_found = 1;
 
 	*lib_local_dpal_args = *local_args;
-	/* NEW */
 	if (pa->lib_ambiguity_codes_consensus) {
 	  PR_ASSERT(dpal_set_ambiguity_code_matrix(lib_local_dpal_args));
 	}
 
 	*lib_local_end_dpal_args = *local_end_args;
-	/* NEW */
 	if (pa->lib_ambiguity_codes_consensus) {
 	  PR_ASSERT(dpal_set_ambiguity_code_matrix(lib_local_end_dpal_args));
 	}
@@ -348,7 +355,12 @@ main(argc,argv)
 	if (NULL != sa->sequence) free(sa->sequence);
 	if (NULL != sa->quality)  free(sa->quality);
 	if (NULL != sa->trimmed_seq) free(sa->trimmed_seq);
+
+	/* edited by T. Koressaar for lowercase masking */
+	if (NULL != sa->trimmed_orig_seq) free(sa->trimmed_orig_seq);
+
 	if (NULL != sa->upcased_seq) free(sa->upcased_seq);
+	if (NULL != sa->upcased_seq_r) free(sa->upcased_seq_r);
 	if (NULL != sa->sequence_name) free(sa->sequence_name);
 	if (NULL != sa->error.data) free(sa->error.data);
 	if (NULL != sa->warning.data) free(sa->warning.data);
@@ -505,6 +517,10 @@ add_must_use_warnings(sa, text, stats)
   if (stats->stability) pr_append_w_sep(&s, sep, "High 3' stability");
   if (stats->no_orf) pr_append_w_sep(&s, sep, "Would not amplify any ORF");
 
+  /* edited by T. Koressaar for lowercase masking: */
+  if (stats->gmasked)
+    pr_append_w_sep(&s, sep, "Masked with lowercase letter");
+
   if (s.data) {
     pr_append_new_chunk(&sa->warning, text);
     pr_append(&sa->warning, " is unacceptable: ");
@@ -520,15 +536,26 @@ oligo_pair_seen(pair, retpair)
     const primer_pair *pair;
     const pair_array_t *retpair;
 {
-    const primer_pair *q = &retpair->pairs[0], 
-		      *stop = &retpair->pairs[retpair->num_pairs];
-    for (; q < stop; q++) {
-	if (q->left->start == pair->left->start
-	    && q->left->length == pair->left->length
-	    && q->right->start == pair->right->start
-	    && q->right->length == pair->right->length) return 1;
-    }
+  const primer_pair *q, *stop;
+
+  /* OLD CODE: const primer_pair *q = &retpair->pairs[0], 
+   *stop = &retpair->pairs[retpair->num_pairs]; */
+
+  /* retpair might not have any pairs in it yet (add_pair
+     allocates memory for retpair->pairs. */
+  if (retpair->num_pairs == 0)
     return 0;
+
+  q = &retpair->pairs[0];
+  stop = &retpair->pairs[retpair->num_pairs];  
+  
+  for (; q < stop; q++) {
+    if (q->left->start == pair->left->start
+	&& q->left->length == pair->left->length
+	&& q->right->start == pair->right->start
+	&& q->right->length == pair->right->length) return 1;
+  }
+  return 0;
 }
 
 /* Add 'pair' to 'retpair'. */
@@ -835,7 +862,6 @@ oligo_param(pa, h, l, local_args, end_args, local_end_args, sa, stats)
     h->ok = OV_UNINITIALIZED;
     h->target = h->gc_content = h->num_ns=h->excl=0;
 
-    /* NEW */
     h->template_mispriming = h->template_mispriming_r = ALIGN_SCORE_UNDEF;
 
     PR_ASSERT(OT_LEFT == l || OT_RIGHT == l || OT_INTL == l);
@@ -845,6 +871,21 @@ oligo_param(pa, h, l, local_args, end_args, local_end_args, sa, stats)
 
     PR_ASSERT(k >= 0);
     PR_ASSERT(k < TRIMMED_SEQ_LEN(sa));
+   
+    /* edited by T. Koressaar for lowercase masking */
+    if(pa->lowercase_masking==1) {
+      if(l==OT_LEFT) {
+	 check_if_lowercase_masked(k, sa->trimmed_orig_seq,h);
+      }
+      if(l==OT_RIGHT) {
+	 check_if_lowercase_masked(j, sa->trimmed_orig_seq,h);
+      }
+      if(h->ok==OV_GMASKED) {
+	 stats->gmasked++;
+	 if (!must_use) return;
+      }
+    }
+    /* end T. Koressar's changes */
 
     gc_and_n_content(j, k-j+1, sa->trimmed_seq, h);
 
@@ -922,18 +963,18 @@ oligo_param(pa, h, l, local_args, end_args, local_end_args, sa, stats)
     }
     if(pa->gc_clamp != 0){
        if(OT_LEFT == l){
-	   for(i=k-pa->gc_clamp+1; i<= k; i++)if(seq[i] !='G'&&seq[i] !='C'){
-	       h->ok = OV_GC_CLAMP;
-	       stats->gc_clamp++;
-	       if (!must_use) return; else break;
-           }
+	 for(i=k-pa->gc_clamp+1; i<= k; i++)if(seq[i] !='G'&&seq[i] !='C'){
+	   h->ok = OV_GC_CLAMP;
+	   stats->gc_clamp++;
+	   if (!must_use) return; else break;
+	 }
        }
        if(OT_RIGHT == l){
-	   for(i=j; i<j+pa->gc_clamp; i++)if(seq[i] != 'G' && seq[i] != 'C'){
-	       h->ok = OV_GC_CLAMP;
-	       stats->gc_clamp++;
-	       if (!must_use) return; else break;
-           }
+	 for(i=j; i<j+pa->gc_clamp; i++)if(seq[i] != 'G' && seq[i] != 'C'){
+	   h->ok = OV_GC_CLAMP;
+	   stats->gc_clamp++;
+	   if (!must_use) return; else break;
+	 }
        }
     }
             
@@ -976,10 +1017,20 @@ oligo_param(pa, h, l, local_args, end_args, local_end_args, sa, stats)
      }
 
     _pr_substr(seq,j,k-j+1,s1);
-    if(OT_LEFT == l || OT_RIGHT == l) 
-      h->temp = seqtm(s1, pa->dna_conc, pa->salt_conc, MAX_NN_TM_LENGTH);
-    else
-      h->temp = seqtm(s1, pa->io_dna_conc, pa->io_salt_conc, MAX_NN_TM_LENGTH);
+                   
+   if(OT_LEFT == l || OT_RIGHT == l)
+     h->temp 
+     = seqtm(s1, pa->dna_conc, pa->salt_conc, pa->divalent_conc, pa->dntp_conc, 
+	     MAX_NN_TM_LENGTH,
+	     pa->tm_santalucia,
+	     pa->salt_corrections); 
+   else
+     h->temp
+     = seqtm(s1, pa->io_dna_conc, pa->io_salt_conc, pa->io_divalent_conc, pa->io_dntp_conc,
+	     MAX_NN_TM_LENGTH,
+	     pa->tm_santalucia,
+	     pa->salt_corrections);
+         
     if (((l == OT_LEFT || l == OT_RIGHT) && h->temp < pa->min_tm)
 	|| (l==OT_INTL && h->temp<pa->io_min_tm)) {
 	h->ok = OV_TM_LOW;
@@ -993,7 +1044,8 @@ oligo_param(pa, h, l, local_args, end_args, local_end_args, sa, stats)
 	if (!must_use) return;
     }
     if (OT_LEFT == l) {
-      if ((h->end_stability = end_oligodg(s1, 5))
+      if ((h->end_stability = end_oligodg(s1, 5,
+					  pa->tm_santalucia))
 	  > pa->max_end_stability) {
 	h->ok = OV_END_STAB;
 	stats->stability++;
@@ -1001,7 +1053,8 @@ oligo_param(pa, h, l, local_args, end_args, local_end_args, sa, stats)
       }
     } else if (OT_RIGHT == l) {
       _pr_reverse_complement(s1, s1_rev);
-      if ((h->end_stability = end_oligodg(s1_rev, 5))
+      if ((h->end_stability = end_oligodg(s1_rev, 5,
+					  pa->tm_santalucia))
 	  > pa->max_end_stability) {
 	  h->ok = OV_END_STAB;
 	  stats->stability++;
@@ -1634,9 +1687,10 @@ pair_param(pa, sa, m, n, int_num, h, local_args, end_args, local_end_args)
     }
 
     /* Compute product Tm and related parameters; check constraints. */
-    h->product_tm 
-      = long_seq_tm(sa->trimmed_seq, h->left->start,
-		   h->right->start - h->left->start + 1, pa->salt_conc);
+   h->product_tm 
+     = long_seq_tm(sa->trimmed_seq, h->left->start,
+		   h->right->start - h->left->start + 1, pa->salt_conc, pa->divalent_conc, pa->dntp_conc);
+      
     PR_ASSERT(h->product_tm != OLIGOTM_ERROR);
 
     min_oligo_tm 
@@ -2074,7 +2128,6 @@ boulder_print_pairs(prog_args, pa, sa, best_pairs)
 	printf("PRIMER_RIGHT%s_END_STABILITY=%.4f\n",
 	       suffix, rev->end_stability);
 
-	/* NEW */
 	if (oligo_max_template_mispriming(fwd) != ALIGN_SCORE_UNDEF)
 	  printf("PRIMER_LEFT%s_TEMPLATE_MISPRIMING=%.4f\n", suffix,
 		 oligo_max_template_mispriming(fwd)
@@ -2201,7 +2254,6 @@ boulder_print_oligos(pa, sa, n, l)
 	    printf("%s%s_END_STABILITY=%.4f\n", type, suffix,
 		    oligo[i].end_stability);
 
-	/* NEW */
 	if (oligo_max_template_mispriming(&oligo[i]) != ALIGN_SCORE_UNDEF)
 	  printf("%s%s_TEMPLATE_MISPRIMING=%.2f\n", type, suffix,
 		 oligo_max_template_mispriming(&oligo[i])
@@ -2298,7 +2350,9 @@ print_explain(stat, l)
     if (stat->stability) printf(",high 3' stability %d", stat->stability);
     if (stat->template_mispriming) printf(",high template mispriming score %d",
 					  stat->template_mispriming);
-
+    /* edited by T. Koressaar for lowercase masking */
+    if(stat->gmasked) printf(",lowercase masking of 3' end %d",stat->gmasked);
+   
     printf(", ok %d\n", stat->ok);
 }
 
@@ -2381,17 +2435,17 @@ set_dpal_args(a)
     memset(a, 0, sizeof(*a));
     for (i = 0; i <= UCHAR_MAX; i++)
 	for (j = 0; j <= UCHAR_MAX; j++)
-	    if (('A' == i || 'C' == i || 'G' == i || 'T' == i || 'N' == i)
-		&& ('A' == j || 'C' == j || 'G' == j || 'T' == j 
-		    || 'N' == j)) {
-		    if (i == 'N' || j == 'N') 
-			a->ssm[i][j] = -25;
-		    else if (i == j)
-			a->ssm[i][j] = 100;
-		    else 
-			a->ssm[i][j] = -100;
-		} else
-		    a->ssm[i][j] = INT_MIN;
+	  if (('A' == i || 'C' == i || 'G' == i || 'T' == i || 'N' == i)
+	      && ('A' == j || 'C' == j || 'G' == j || 'T' == j 
+		  || 'N' == j)) {
+	    if (i == 'N' || j == 'N') 
+	      a->ssm[i][j] = -25;
+	    else if (i == j)
+	      a->ssm[i][j] = 100;
+	    else 
+	      a->ssm[i][j] = -100;
+	  } else
+	    a->ssm[i][j] = INT_MIN;
 
     a->gap                = -200;
     a->gapl               = -200;
@@ -2573,14 +2627,6 @@ oligo_mispriming(h, pa, sa, l, align_args)
     ? h->start + h->length - 1
     : h->start;
 
-  /* NOT USED
-  if (pa->short_match && pa->short_match < h->length) {
-    match_length = pa->short_match;
-    if (!warned) 
-      fprintf(stderr, "Using short_match of %d\n", pa->short_match);
-    warned = 1;
-  } else
-  */
   match_length = h->length;
 
   _pr_substr(sa->trimmed_seq, first, h->length, s_tmp);
@@ -2666,7 +2712,11 @@ oligo_mispriming(h, pa, sa, l, align_args)
     tmp_char = target[first_untrimmed];
     target[first_untrimmed] = '\0';
 
-    tmp_score = align(oseq, target, align_args);
+    if (strlen(target) < 3) {
+      tmp_score = 3;
+    } else {
+      tmp_score = align(oseq, target, align_args);
+    }
 
     if (debug) {
       if (l == OT_LEFT) fprintf(stderr, "\n************ OLIGO = LEFT\n");
@@ -2694,7 +2744,7 @@ oligo_mispriming(h, pa, sa, l, align_args)
 
     /* 4. Align to the reverse strand of the template. */
     h->template_mispriming_r
-      = align(oseq, target_r, align_args); /* lib_local_end_dpal_args); */
+      = align(oseq, target_r, align_args);
 
     if (debug)
       fprintf(stderr, "other strand Score %d aligning %s against %s\n\n", 
@@ -2804,43 +2854,8 @@ find_stop_codon(s, start, direction)
   return -1;
 }
 
-/* =========================================================== */
-/* Various fail-stop wrappers for standard library functions.  */
-
-void *
-pr_safe_malloc(x)
-    size_t x;
-{
-    void *r = malloc(x);
-    if (NULL == r) OOM_ERROR;
-    return r;
-}
-
-void *
-pr_safe_realloc(p, x)
-    void *p;
-    size_t x;
-{
-    void *r = realloc(p, x);
-    if (NULL == r) OOM_ERROR;
-    return r;
-}
-
-static FILE *
-safe_fopen(path, mode)
-    const char *path, *mode;
-{
-    FILE *r = fopen(path, mode);
-    if (NULL == r) {
-	fprintf(stderr, "%s: unable to open file %s:",
-		pr_program_name, path);
-	perror("");
-	exit (-1);
-    }
-    return r;
-}
-
-int strcmp_nocase(s1, s2)
+int
+strcmp_nocase(s1, s2)
 char *s1, *s2;
 {
    static char M[UCHAR_MAX];
@@ -2899,5 +2914,57 @@ int n_f, n_r, n_m;
      { free(mid[i].repeat_sim.score); mid[i].repeat_sim.score = NULL; }
 }
 
+/*  Edited by T. Koressaar for lowercase masking. This function checks
+ if the 3' end of the primer has been masked by lowercase letter.
+ Function created/Added by Eric Reppo, July 9, 2002
+ */
+static void
+check_if_lowercase_masked(position, sequence, h)
+     const int position;
+     const char *sequence;
+     primer_rec *h;
+{   
+   const char* p = &sequence[position];
+   if ('a' == *p || 'c' == *p ||'g' == *p || 't' == *p) {
+      h->ok=OV_GMASKED;
+   }
+}
 
 /* =========================================================== */
+/* Various fail-stop wrappers for standard library functions.  */
+
+void *
+pr_safe_malloc(x)
+    size_t x;
+{
+    void *r = malloc(x);
+    if (NULL == r) OOM_ERROR;
+    return r;
+}
+
+void *
+pr_safe_realloc(p, x)
+    void *p;
+    size_t x;
+{
+    void *r = realloc(p, x);
+    if (NULL == r) OOM_ERROR;
+    return r;
+}
+
+static FILE *
+safe_fopen(path, mode)
+    const char *path, *mode;
+{
+    FILE *r = fopen(path, mode);
+    if (NULL == r) {
+	fprintf(stderr, "%s: unable to open file %s:",
+		pr_program_name, path);
+	perror("");
+	exit (-1);
+    }
+    return r;
+}
+
+/* End of fail-stop wrappers. */
+/* =========================================================== */
